library IEEE;
use IEEE.STD_LOGIC_1164.ALL;
use IEEE.STD_LOGIC_ARITH.ALL;
use IEEE.STD_LOGIC_UNSIGNED.ALL;


ENTITY ccd_logic IS
GENERIC ( CLK_COUNT		: natural	:= 958;
          PIX_COUNT		: natural	:= 2049;
          PIX_OFFSET		: natural	:= 33;
          ROG_OFFSET		: natural	:= 40;
          ADC_COUNT		: natural	:= 100;
          ADC_OFFSET		: natural	:= 25;
          RESET_COUNT 	: natural 	:= 95;
          RESET_OFFSET	: natural	:= 3; 
          INT1_COUNT		: natural	:= 2);
    PORT (	CLK		:	in		bit;
				INT0		:	out	bit;	-- "RESET" Interrupt
				INT1		:	out	bit;	-- "READ"  Interrupt 
				RESET		:	out	bit;	-- Reset fr Shiftregs und Readcounter
				ADC_RDY	:	out	bit;	-- read ADC
				CCD_CLK	: 	out	bit;
				CCD_ROG	: 	out	bit);
END ccd_logic;


ARCHITECTURE Behavioral OF ccd_logic IS


-- die xxx_EN-Signale sorgen fr das richtige Timing und werden durchgehend erzeugt
-- die xxx_ACT-Signale sorgen fr die Freigabe zum richtigen Zeitpunkt(Pixel)


SIGNAL ROG_ACT		:	BIT							:='0';
SIGNAL ADC_ACT		:	BIT							:='0';
SIGNAL INT_ACT		:	BIT							:='0';
SIGNAL ROG_EN 		:	BIT							:='0'; 
SIGNAL ADC_EN	 	:	BIT							:='0';
SIGNAL INT_EN   	:	BIT                     :='0';
SIGNAL INT_SET  	:	BIT                     :='0';
SIGNAL INT_RES  	:	BIT                     :='0';
SIGNAL RESET_EN	:	BIT							:='0';
SIGNAL PIX_CLK		:	BIT                     :='0';
SIGNAL PIX_CNT		:	integer range 0 to 2088	:= 0;
SIGNAL CLK_CNT		:	integer range 0 to 958	:= 0;


BEGIN


CCD_CLK	<= PIX_CLK;
CCD_ROG	<= NOT (ROG_EN AND ROG_ACT);
RESET		<= RESET_EN AND ADC_ACT;
INT1		<= INT_EN AND INT_ACT;
ADC_RDY	<= ADC_EN AND ADC_ACT;


--	Oszillatorfreq. /	CCD-Pixel * Abtastrate
--		  20MHz		    /	 ( 2088 	 *    ( 5     * 2 ) ) = 958 (ca.)

--	Zhler fr die 20MHz Takte
CLK_COUNTER: PROCESS (CLK)
BEGIN
	IF CLK = '0' AND CLK'event THEN
		IF CLK_CNT = CLK_COUNT THEN
			CLK_CNT <= 0;
		ELSE
			CLK_CNT <= CLK_CNT +1;
		END IF;
	END IF;	
END PROCESS CLK_COUNTER;


-- Takt-Erzeuger fr CCD-Takte (20,88kHz)
CLK_PRESCALER: PROCESS (CLK_CNT)
VARIABLE TEMP : BIT;
BEGIN
TEMP := PIX_CLK;
	IF CLK_CNT = 0 THEN
		PIX_CLK <= NOT TEMP;
	END IF;
END PROCESS CLK_PRESCALER;


--	Zhler fr die CCD Takte (10,44kHz)
PIX_COUNTER: PROCESS (PIX_CLK)
BEGIN
	IF PIX_CLK = '1' AND PIX_CLK'event THEN
		IF PIX_CNT = 2088 THEN
			PIX_CNT <= 0;
		ELSE
			PIX_CNT <= PIX_CNT +1;
		END IF;
	END IF;	
END PROCESS PIX_COUNTER;

 
SIGNAL_ENABLER: PROCESS (CLK_CNT)
BEGIN
	IF CLK_CNT = (ROG_OFFSET +1) THEN
		ROG_EN	<= '1';
	ELSIF CLK_CNT = RESET_COUNT THEN
		RESET_EN	<= '1';
	ELSIF CLK_CNT = (RESET_COUNT + RESET_OFFSET) THEN
      RESET_EN <= '0';
	ELSIF CLK_CNT = ADC_COUNT THEN
		ADC_EN	<= '1';
	ELSIF CLK_CNT = (ADC_COUNT + ADC_OFFSET) THEN
		ADC_EN	<= '0';
	ELSIF CLK_CNT = (CLK_COUNT - ROG_OFFSET) THEN
		ROG_EN	<= '0';
	END IF;	
END PROCESS SIGNAL_ENABLER;


INT1_ENABLER: PROCESS (CLK_CNT)
BEGIN
   IF INT_SET = '1' THEN
      INT_EN <= '1';
	ELSIF INT_RES = '1' THEN
      INT_EN <= '0';
	END IF;	
END PROCESS INT1_ENABLER;


INT1_START: PROCESS (CLK_CNT)
BEGIN
   IF PIX_CLK = '1' THEN
	   IF CLK_CNT = (ADC_COUNT -5) THEN
		   INT_SET <= '1';
	   ELSE
		   INT_SET <= '0';
	   END IF;
	END IF;	
END PROCESS INT1_START;


INT1_STOP: PROCESS (CLK_CNT)
BEGIN
   IF PIX_CLK = '0' THEN
	   IF CLK_CNT = (CLK_COUNT -5) THEN
		   INT_RES <= '1';
	   ELSE
		   INT_RES <= '0';
	   END IF;
   END IF;	
END PROCESS INT1_STOP;


-- Aktiviert den Interrupt 0
INT0_ACTIVATOR: PROCESS (PIX_CNT, PIX_CLK)
BEGIN
   IF PIX_CNT = 1 THEN
      INT0 <= '1';
   ELSIF PIX_CNT = (PIX_OFFSET -1) THEN
      INT0 <= '0';
   END IF;
END PROCESS INT0_ACTIVATOR;


-- Aktiviert den Interrupt 1
INT1_ACTIVATOR: PROCESS (PIX_CLK)
BEGIN
   IF PIX_CLK = '1' AND PIX_CLK'event THEN
	   IF PIX_CNT = (PIX_OFFSET +1) THEN
		   INT_ACT <= '1';
	   ELSIF PIX_CNT = (PIX_COUNT + PIX_OFFSET) THEN
		   INT_ACT <= '0';
      END IF;
	END IF;	
END PROCESS INT1_ACTIVATOR;


-- Aktiviert das ROG Signal fr die CCD-Zeilen
ROG_ACTIVATOR: PROCESS (PIX_CNT, PIX_CLK)
BEGIN
	IF PIX_CNT = 1 AND PIX_CLK = '1' THEN
		ROG_ACT	<= '1';
   ELSE 
		ROG_ACT	<= '0';
	END IF;	
END PROCESS ROG_ACTIVATOR;


-- Aktiviert das Sample Signal fr die AD-Wandler
ADC_ACTIVATOR: PROCESS (PIX_CNT, PIX_CLK)
BEGIN
   IF PIX_CLK = '1' THEN
	   IF PIX_CNT > PIX_OFFSET THEN
         IF PIX_CNT < (PIX_OFFSET + PIX_COUNT) THEN
			   ADC_ACT <= '1';
         END IF;
		END IF;
	ELSE
	   ADC_ACT <= '0';
	END IF;
END PROCESS ADC_ACTIVATOR;


END Behavioral;


